'use strict';

exports.__esModule = true;
exports.messageAdd = undefined;
exports.messageReceive = messageReceive;
exports.popMessages = popMessages;
exports.messageSend = messageSend;

var _actionTypes = require('../actionTypes');

var _index = require('../themes/default/components/TypingIndicator/index');

var _index2 = _interopRequireDefault(_index);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Action creator: Adds a message to the store.
 * @type Redux action creator
 * @param {object} message
 * @return {{type: MESSAGE_ADD}} redux action type returned
 */
var messageAdd = exports.messageAdd = function messageAdd(message) {
    return {
        type: _actionTypes.MESSAGE_ADD,
        payload: message
    };
};

var popping = false;
var waitingForMessage = false;

/**
 * Async action creator:
 * Dispatched when a message is received from the server. The message will get added to a dispatch queue
 * if not the first message. The message queue is then popped off into the message store based on the delays given.
 * @param {object} message
 * @return {function()}
 */
function messageReceive(message) {
    return function (dispatch, getState) {
        var _getState$messages = getState().messages,
            messages = _getState$messages.messages,
            messageQueue = _getState$messages.messageQueue;
        var _getState$config$typi = getState().config.typingStatus,
            active = _getState$config$typi.active,
            baseDelay = _getState$config$typi.baseDelay,
            variance = _getState$config$typi.variance,
            letterDelay = _getState$config$typi.letterDelay,
            minDelay = _getState$config$typi.minDelay,
            maxDelay = _getState$config$typi.maxDelay;

        var queueDelay = 0;

        if (active) {
            // Base delay plus the variance.
            queueDelay = Math.round(baseDelay + variance * Math.round(1.5 + Math.random() * -3));
            // Add Delay per letter in the message.
            var textSize = 0;
            message.pages.map(function (page) {
                textSize += (page.title ? page.title.length : 0) + (page.text ? page.text.length : 0);
            });
            queueDelay += textSize * 40;
            // Clamp to min and max delay size.
            queueDelay = Math.max(Math.min(queueDelay, maxDelay), minDelay);
        }

        // Catch following messages or add first message.
        if (messageQueue.length === 0 && (!waitingForMessage || Date.now() - messages[messages.length - 1].timeAdded > queueDelay)) {
            dispatch(messageAdd(message));
        } else {
            waitingForMessage = false;
            dispatch({
                type: _actionTypes.MESSAGE_QUEUE,
                payload: {
                    message: message,
                    delay: queueDelay
                }
            });
            if (!popping) {
                popping = true;
                dispatch(popMessages());
            }
        }
        dispatch({
            type: _actionTypes.MESSAGE_RECEIVE,
            payload: message
        });
    };
}

function popMessages() {
    return function (dispatch, getState) {
        var messageQueue = getState().messages.messageQueue;
        var delay = function delay() {
            return new Promise(function (resolve) {
                return setTimeout(function () {
                    dispatch(messageAdd(messageQueue[0].message));
                    resolve();
                }, messageQueue[0].delay);
            });
        };

        delay().then(function () {
            var messageQueue = getState().messages.messageQueue;
            if (messageQueue.length > 0) {
                dispatch(popMessages());
            } else {
                popping = false;
            }
        });
    };
}

/**
 * Async action creator:
 * Dispatched when a message is sent by the user.
 * @param {object} message
 * @return {function()} dispatches {@link messageAdd} action creator and `MESSAGE_SEND` action type
 */
function messageSend(_ref) {
    var postback = _ref.postback,
        text = _ref.text,
        type = _ref.type;

    var message = {
        type: type || 'text',
        origin: 'local',
        postback: postback,
        text: text
    };
    return function (dispatch) {
        waitingForMessage = true;
        dispatch({
            type: _actionTypes.MESSAGE_SEND,
            payload: message
        });
        dispatch(messageAdd(message));
    };
}